# To customize the measurement (ruler) menu, save a copy of this script in $HOME/.qub-express/MeasurementScripts/

# The following function is called in the main thread, prior to measuring;  you can return identical or modified args.
#    * segments: list of qubx.data_types.SourceSeg, one per measurement
#    * table: destination of measurements
#    * row_offset: first row index to put measurements in
def before_measure(segments, table, row_offset):
    return hill_before_measure(segments, table, row_offset)
    # return segments, table, row_offset

# The following function is called on each selection to be measured.
#    * file: qubx.data_types.QubData
#    * signals: list representing each data signal; provides:
#      signals[i].name: e.g. "Current"
#      signals[i].get_samples(): returns numpy.array(dtype=float32)
#      signals[i].hidden: True if user has put the signal offscreen
#    * source_signal: the entry in signals which is chosen in Data Source
#    * inclusion_mask: numpy.array(dtype=bool): whether each sample is included (True) or excluded (False)
#    * measured: a dictionary to store results in
# It is called in a worker thread, so please don't interact with the rest of the program directly (use gobject.idle_add).

def measure_selection(segment, signals, source_signal, inclusion_mask, measured):
    for signal in [signals[hill_sig_I], signals[hill_sig_L]]:
        # use the built-in measurements:
        qubx.dataGTK.measure_selection_one_signal(segment, signal, inclusion_mask, measured)

# The following function is called in the main thread, after a measurement sequence is completed.
#    * segments: list of qubx.data_types.SourceSeg, one per measurement
#    * table: qubx.table.Table containing results
#    * row_offset: first row index of results in table
def after_measure(segments, table, row_offset):
    hill_after_measure(segments, table, row_offset)


hill_sig_I = hill_sig_L = 0 # current and ligand signal indices, shared among hill_*()

def hill_before_measure(segments, table, row_offset):
    # pick out current and ligand signals, so we don't have to measure any others
    global hill_sig_I, hill_sig_L
    # assume current is in chosen "DataSource" signal
    hill_sig_I = QubX.DataSource.signal
    # default ligand signal, if not given from model:
    hill_sig_L = hill_sig_I + 1
    if hill_sig_L >= segments[0].file.signals.size:
        if hill_sig_I:
            hill_sig_L = 0
        else:
            print 'before_measure: no stimulus signal found.'
            return [], table, row_offset
    # find ligand name in model, arbitrarily picking first in Rates table if more than one ligand
    ligands = [row.Ligand for row in QubX.Models.file.rates if row.Ligand]
    if ligands:
        ligand = ligands[0]
        # look up stimulus -> signal mapping
        stimuli = [row.Signal_index
                   for row in segments[0].file.stimuli
                   if (row.Name == ligand) and (row.Type == STIM_TYPE_SIGNAL)]
        if stimuli:
            hill_sig_L = stimuli[0]
    print 'hill_sig_L = ',hill_sig_L
    return segments, table, row_offset

def hill_after_measure(segments, table, row_offset):
    """Example: Fit measurements to the Hill equation.  log and normalize measurements, show a plot, and fit it."""
    name_I = segments[0].file.signals[hill_sig_I, 'Name']
    name_L = segments[0].file.signals[hill_sig_L, 'Name']
    # augment the prior measurements by taking log10(ligand) and normalizing current to 1.0
    Iseq = [row.__dict__['%s Mean' % name_I] for row in table]
    Imax = max(Iseq)
    if Imax <= 0: # peak is probably downward
        Imax = min(Iseq)
    if not Imax:
        Imax = 1
    n = len(segments)
    for i in xrange(len(segments)):
        mean = table[row_offset+i, '%s Mean'%name_L]
        table[row_offset+i, 'log10(%s Mean)'%name_L] = log10(mean) if (mean > 0.0) else 0.0
        table[row_offset+i, '%s Normalized Peak'%name_I] = table[row_offset+i, '%s Peak'%name_I] / Imax

    # show a Chart and fit with the Hill equation:
    if len(segments) > 1:
        plot = QubX.Charts.add_two_plot(table.label, 'log10(%s Mean)'%name_L, '%s Normalized Peak'%name_I, False, False)
        QubX.show_charts()
        doze(1) # plot may need a second to set up?

        plot.controls.robot.set_expr('(x**Hill)/(x**Hill + EC50**Hill)')
        plot.layerset = plot.controls
        plot.controls.robot.set_param('Hill', 2.0)
        plot.controls.robot.set_param('EC50', 2.0)
        plot.controls.robot.fit()
        plot.controls.robot.sync()

        ## options for output:
        ##   console (Admin:Scripts):
        #print plot.nbParams
        #print plot.controls.param_vals[0], '+/-', plot.controls.std_err_est
        #print 'R^2:', plot.controls.r2

        ##   add a line to a table of results:
        #result_table = QubX.Tables.find_table('Hill results') or QubX.Tables.add_user_table('Hill results', initial_row=False)
        #row = {}
        #for pname, pval, err in izip(plot.controls.param_names, plot.controls.param_vals, plot.controls.std_err_est):
        #    row[pname] = pval
        #    row['%s std' % pname] = err
        #result_table.append(row)
        
        ##   send notebook items:
        #Notebook.send(qubx.notebook.NbText('Fitted by the Hill measurement script:'))
        #Notebook.send(plot.nbPicture)
        #Notebook.send(plot.nbChart)
        #Notebook.send(plot.nbParams)
